import tkinter as tk
from tkinter import *
from tkinter import ttk
from GraphProcessing import GraphVis, HistogramVis, PieChartVis, ScatterPlot, BoxPlot


class Application(tk.Tk):
    def __init__(self):
        super().__init__()
        self.textfile = 'description.txt'
        self.title("Spotify Top Tracks Analysis")
        self.create_upper_frame()
        self.create_left_frame()
        self.graph = GraphVis()
        self.hist_graph = HistogramVis()
        self.scatter_graph = ScatterPlot()
        self.pie_chart = PieChartVis()
        self.box_plot = BoxPlot()

    # create an upper frame to store all widgets
    def create_upper_frame(self):
        self.upper_frame = ttk.LabelFrame(self, text="Select Graph Type")
        self.upper_frame.pack(side=RIGHT, expand=FALSE)
        self.create_graph_buttons()
        self.create_buttons()
        self.read_description(self.textfile)

    # create a frame for placing graphs
    def create_left_frame(self):
        self.left_frame = Frame(self)
        # self.left_frame.grid(row=0, column=0)
        self.left_frame.pack(side=RIGHT)
        self.columnconfigure(0, weight=2)
        self.rowconfigure(0, weight=2)

    # create buttons for quit and a label
    def create_buttons(self):
        self.label_select = ttk.Label(self.upper_frame, text='Select Your Parameter')
        self.quitbutton = ttk.Button(self.upper_frame, text="Quit",
                                     command=self.quit)
        self.label_select.grid(row=2, column=1)
        self.quitbutton.grid(row=4, column=4)

    # create graph buttons and activate combobox commands
    def create_graph_buttons(self):
        self.hist_button = ttk.Button(self.upper_frame, text="Histogram",
                                      command=self.create_frame_hist())
        self.pie_button = ttk.Button(self.upper_frame, text="Pie Chart",
                                     command=self.create_frame_pie())
        self.correlation_button = ttk.Button(self.upper_frame, text="Scatter Plot",
                                             command=self.create_frame_scatter())
        self.boxplot_button = ttk.Button(self.upper_frame, text="Box Plot",
                                         command=self.create_box_plot())
        self.hist_button.grid(row=1, column=1, padx=2)
        self.pie_button.grid(row=1, column=2, padx=2)
        self.correlation_button.grid(row=1, column=3, padx=2)
        self.boxplot_button.grid(row=1, column=4, padx=2)

    # create histogram frame and combobox
    def create_frame_hist(self):
        n = tk.StringVar()
        columns1 = ['popularity', 'duration_ms', 'duration_ms', 'danceability',
                         'energy', 'key', 'loudness', 'speechiness', 'acousticness',
                         'instrumentalness', 'liveness', 'valence', 'tempo',
                         'time_signature']
        self.cb_hist = ttk.Combobox(self.upper_frame, values=columns1,
                                    state="readonly", textvariable=n)
        self.cb_hist.bind("<<ComboboxSelected>>", self.binding_hist)
        self.cb_hist.current(0)
        self.cb_hist.grid(row=3, column=1)

    # used for binding event with combobox and plot histograms
    def binding_hist(self, event):
        x = self.cb_hist.get()
        self.hist_graph.plot(x, self.left_frame)

    # create box plot frame and combobox
    def create_box_plot(self):
        n = tk.StringVar()
        columns4 = ['popularity', 'duration_ms', 'duration_ms', 'danceability',
                         'energy', 'key', 'loudness', 'speechiness', 'acousticness',
                         'instrumentalness', 'liveness', 'valence', 'tempo',
                         'time_signature']
        self.cb_bp = ttk.Combobox(self.upper_frame, values=columns4,
                                    state="readonly", textvariable=n)
        self.cb_bp.bind("<<ComboboxSelected>>", self.binding_bp)
        self.cb_bp.current(0)
        self.cb_bp.grid(row=3, column=4)

    # used for binding event with combobox and plot box plots
    def binding_bp(self, event):
        z = self.cb_bp.get()
        self.box_plot.plot(z, self.left_frame)

    # create scatter plot frame and combobox
    def create_frame_scatter(self):
        columns3 = ['duration_ms', 'duration_ms', 'danceability',
                         'energy', 'key', 'loudness', 'speechiness', 'acousticness',
                         'instrumentalness', 'liveness', 'valence', 'tempo',
                         'time_signature']
        self.cb_y_scatter = ttk.Combobox(self.upper_frame,
                                         values=columns3, state="readonly")
        self.cb_y_scatter.bind("<<ComboboxSelected>>", self.binding_scatter)
        self.cb_y_scatter.grid(row=3, column=3)

    # used for binding event with combobox and plot scatter plots
    def binding_scatter(self, event):
        y = self.cb_y_scatter.get()
        self.scatter_graph.plot(y, self.left_frame)

    # create pie chart frame and combobox
    def create_frame_pie(self):
        columns2 = ['explicit', 'key', 'mode', 'track_genre', 'time_signature']
        self.cb_pie = ttk.Combobox(self.upper_frame, values=columns2)
        self.cb_pie.bind("<<ComboboxSelected>>", self.binding_pie)
        self.cb_pie.grid(row=3, column=2)

    # used for binding event with combobox and plot pie charts
    def binding_pie(self, event):
        para = self.cb_pie.get()
        self.pie_chart.plot(para, self.left_frame)

    # read information from text file describing each parameter's meaning and show it
    def read_description(self, text):
        with open(text) as t:
            para_text = t.read()
        self.message_frame = LabelFrame(self.upper_frame, text='Meaning of each parameter')
        self.message_frame.grid(row=5, column=1, columnspan=4, sticky=EW)
        self.para_message = tk.Message(self.message_frame, font='Gibson', text=para_text)
        self.para_message.pack(side=LEFT)

    # def create_statistics_frame(self):
    #     self.stat_frame = LabelFrame(self, text='Descriptive Stats')
    #     self.mean_frame = LabelFrame(self.stat_frame, text='Mean')
    #     self_mean_entry = Entry


    def run(self):
        self.mainloop()
