import matplotlib.pyplot as plt
import numpy as np
from DataProcessing import SpotifyData
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg


# create class for other class to inherit
class GraphVis:
    def __init__(self):
        self.load_data = SpotifyData()
        self.data = self.load_data.df.copy()

    def plot(self, **args):
        pass

    def destroy(self):
        pass


# For plotting piecharts
class PieChartVis(GraphVis):
    def __init__(self):
        super().__init__()

    def plot(self, para, frame):
        self.fig3 = plt.Figure(figsize=(6, 5), dpi=100)
        self.ax3 = self.fig3.add_subplot(111)
        self.pop_50 = self.data[self.data.popularity > 50]
        self.pie_df = self.pop_50.groupby(para).size()
        self.pie_chart = FigureCanvasTkAgg(self.fig3, master=frame)
        self.pie_chart.get_tk_widget().grid(row=0, column=0, rowspan=5)
        self.pie_df.plot(kind='pie', labels=self.pie_df.index,
                         autopct='%1.1f%%', ax=self.ax3)
        self.ax3.set_title(f"{para} proportions pie chart")

    # for destroying the graph
    def destroy(self):
        self.pie_chart.get_tk_widget().destroy()


# For plotting histograms
class HistogramVis(GraphVis):
    def __init__(self):
        super().__init__()

    def plot(self, x_axis, frame):
        self.fig1 = plt.Figure(figsize=(6, 5), dpi=100)
        self.ax1 = self.fig1.add_subplot(111)
        self.hist_df = self.data[x_axis]
        self.hist_bar = FigureCanvasTkAgg(self.fig1, master=frame)
        self.hist_bar.get_tk_widget().grid(row=0, column=0)
        self.hist_df.plot(kind='hist', legend=True, ax=self.ax1)
        self.ax1.set_title(f"{x_axis} Histogram")
        self.ax1.set_xlabel(x_axis)
        self.ax1.set_ylabel("Frequency")

    # for destroying the graph
    def destroy(self):
        self.hist_bar.get_tk_widget().destroy()


# For plotting scatter plots
class ScatterPlot(GraphVis):
    def __init__(self):
        super().__init__()

    def plot(self, y_axis, frame):
        self.fig2 = plt.Figure(figsize=(6, 5), dpi=100)
        self.ax2 = self.fig2.add_subplot(111)
        self.pop_50 = self.data[self.data.popularity > 50]
        self.ax2.scatter(self.pop_50['popularity'], self.pop_50[y_axis], color='lightblue')
        self.scatter = FigureCanvasTkAgg(self.fig2, master=frame)
        self.scatter.get_tk_widget().grid(row=0, column=0)
        self.ax2.axhline(y=np.nanmean(self.data[y_axis]), color='red', linestyle='--', label='Average')
        self.ax2.legend([y_axis])
        self.ax2.set_xlabel('Popularity')
        self.ax2.set_title(f'Correlation between Popularity and {y_axis}')

    # for destroying the graph
    def destroy(self):
        self.scatter.get_tk_widget().destroy()


# For plotting box plots
class BoxPlot(GraphVis):
    def __init__(self):
        super().__init__()

    def plot(self, para, frame):
        self.fig = plt.Figure(figsize=(6, 5), dpi=100)
        self.ax2 = self.fig.add_subplot(111)
        self.ax2.boxplot(self.data[para])
        self.boxplot = FigureCanvasTkAgg(self.fig, master=frame)
        self.boxplot.get_tk_widget().grid(row=0, column=0)
        self.ax2.set_xlabel(para)
        self.ax2.set_title(f'{para} Box Plot')

    # for destroying the graph
    def destroy(self):
        self.boxplot.get_tk_widget().destroy()
